@@ -5,6 +5,9 @@ import json
 import ast
 import jwt
 import urllib #Needed for avoiding using requests, which modified the S3 presigned URL
+from urllib.parse import urlparse, parse_qs
+import csv
+import pathlib
 
 from .utils import convert_timestamps, send_post_request, send_get_request, decode_jwt_token, querystr_to_json, att2statement
 from .lineage_graph import create_lineage_graph 
@@ -29,10 +32,10 @@ class ScribeClient:
     scribe_findings_dataset = SCRIBE_FINDINGS_DATASET
 
     def get_chache_data(self):
+        # Exclude product_list from cache as per requirements
         return {
             "timestamp": datetime.datetime.now().isoformat(),
             "datasets": self.datasets,
-            "product_list": self.product_list,
             "team_name": self.team_name,
             "team_info": self.team_info
         }
@@ -46,18 +49,18 @@ class ScribeClient:
                 logging.error(f"Error parsing timestamp: {e}")
                 return True
             return (datetime.datetime.now() - last_update).days > 1
+        
+    def __init__(self, api_token=None, ignore_env = False, base_url="https://api.scribesecurity.com", refresh_interval=600, cached_data=None, env = "prod",
+             username=None, password=None):
+    
 
-    def __init__(self, api_token=None, ignore_env = False, base_url="https://api.scribesecurity.com", refresh_interval=600, cached_data={}, env = "prod"):
         def update_from_cached_data(cached_data):
             if "datasets" in cached_data:
                 self.datasets = cached_data["datasets"]
                 self.dataset_ids = self.create_dataset_id_map(self.datasets)
-            else: 
-                logging.error("No datasets in provided cached data")
-            if "product_list" in cached_data:  
-                self.product_list = cached_data["product_list"]
             else:
-                logging.error("No product list in provided cached data")
+                logging.error("No datasets in provided cached data")
+            # product_list intentionally not loaded from cache
             if "team_name" in cached_data:
                 self.team_name = cached_data["team_name"]
             else:
@@ -67,6 +70,29 @@ class ScribeClient:
             else:
                 logging.error("No team info in provided cached data")
 
+        # Default cache file path
+        cache_dir = os.path.join(os.getcwd(), ".scribe")
+        cache_file = os.path.join(cache_dir, "cache.json")
+        loaded_cache = None
+
+        # Only load cache if not explicitly provided
+        if cached_data is None:
+            try:
+                if os.path.exists(cache_file):
+                    # Check file age
+                    file_mtime = datetime.datetime.fromtimestamp(os.path.getmtime(cache_file))
+                    if (datetime.datetime.now() - file_mtime).days <= 1:
+                        with open(cache_file, "r") as f:
+                            loaded_cache = json.load(f)
+                    else:
+                        logging.info("Cache file is older than one day, will refresh cache.")
+                else:
+                    # Ensure .scribe directory exists
+                    pathlib.Path(cache_dir).mkdir(parents=True, exist_ok=True)
+            except Exception as e:
+                logging.error(f"Error loading cache file: {e}")
+        else:
+            loaded_cache = cached_data
 
         if ignore_env:
             self.api_token = api_token
@@ -75,8 +101,16 @@ class ScribeClient:
         if not self.api_token:
             logging.error("No API token provided. Please set the SCRIBE_TOKEN environment variable or pass it as an argument.")
             return
+        self.username = username
+        self.password = password
         self.env = env
         self.base_url = base_url
+        
+        if env == "prod":
+            self.superset_base_url = "https://superset.scribesecurity.com"
+        else:
+            self.superset_base_url = f"https://internal-superset.{env}.scribesecurity.com"
+        
         self.refresh_interval = refresh_interval
         self.last_refresh = datetime.datetime.now() - datetime.timedelta(days=5)  # Initialize to a time in the past
 
@@ -93,7 +127,8 @@ class ScribeClient:
             self.last_refresh = datetime.datetime.now() # - datetime.timedelta(hours=5)
             self.refresh_data(force=True)
             
-            if not cached_data or self.need_cache_data_update(cached_data):
+            # Use loaded_cache if available and not stale, else refresh and save
+            if not loaded_cache or self.need_cache_data_update(loaded_cache):
                 d = self.get_datasets()
                 if not d:
                     logging.error("Refresh: Failed to update dataset IDs")
@@ -110,8 +145,17 @@ class ScribeClient:
                 t = self.get_team_info()
                 if not t:
                     logging.error("Failed to get team data.")
+
+                # Save cache (excluding product_list)
+                try:
+                    pathlib.Path(cache_dir).mkdir(parents=True, exist_ok=True)
+                    cache_data = self.get_chache_data()
+                    with open(cache_file, "w") as f:
+                        json.dump(cache_data, f, indent=2)
+                except Exception as e:
+                    logging.error(f"Error saving cache file: {e}")
             else:
-                update_from_cached_data(cached_data)
+                update_from_cached_data(loaded_cache)
 
     def validate_dataset(self, dataset_name):
         if not self.dataset_ids:
@@ -1091,4 +1135,331 @@ class ScribeClient:
             r = self.query_superset(self.scribe_findings_dataset, query, time_tokens=["timestamp"])
             if r:
                 return f"{title}\n{self.format_query_result_table(r)}"
-        return "Error running query"
\ No newline at end of file
+        return "Error running query"
+    
+    def superset_authenticate(self):
+        if not self.username or not self.password:
+            logging.error("Username and password must be provided for Superset login.")
+            return False
+
+        login_url = f"{self.superset_base_url}/api/v1/security/login"
+        body = {
+            "username": self.username,
+            "password": self.password,
+            "provider": "db",
+            "refresh": True
+        }
+
+        response = self.send_post_request(login_url, token = None, body=body)
+        if not response:
+            logging.error("Login failed. No response received.")
+            return False
+
+        try:
+            result = response.json()
+        except json.JSONDecodeError:
+            logging.error("Failed to parse login response.")
+            return False
+
+        access_token = result.get("access_token")
+        if access_token:
+            self.jwt_token = access_token
+            self.superset_token = access_token
+            logging.info("Login successful. Tokens obtained.")
+            return True
+        else:
+            logging.error("Login failed. No access token found.")
+            return False
+    
+    def get_dataset_links(self):
+                
+        self.superset_authenticate()
+
+        if not self.jwt_token:
+            logging.error("No JWT token found. Please authenticate first.")
+            return []
+
+        dataset_links = []
+
+        #Use slug to directly fetch the API dashboard's datasets
+        dashboard_slug = "api-dashboard"
+        datasets_url = f"{self.superset_base_url}/api/v1/dashboard/{dashboard_slug}/datasets"
+        logging.info(f"Fetching datasets from {datasets_url}")
+
+        datasets_response = self.send_get_request(datasets_url, token=self.jwt_token)
+        if not datasets_response:
+            logging.error("Failed to fetch datasets for API dashboard.")
+            return []
+
+        try:
+            datasets_data = datasets_response.json()
+            datasets = datasets_data.get("result", [])
+
+            for dataset in datasets:
+                dataset_id = dataset.get("id")
+                if dataset_id:
+                    link = f"{self.base_url}/explore/?datasource_type=table&datasource_id={dataset_id}"
+                    dataset_links.append(link)
+
+        except json.JSONDecodeError as e:
+            logging.error(f"Error parsing dataset response: {e}")
+
+        logging.info(f"Total dataset links collected: {len(dataset_links)}")
+        return dataset_links
+
+
+    def get_dataset_data(self, dataset_links, output_file="dataset_metadata.csv"):
+        records = []
+
+        for link in dataset_links:
+            #Properly parse dataset ID from the URL
+            parsed_url = urlparse(link)
+            query_params = parse_qs(parsed_url.query)
+            dataset_id_list = query_params.get("datasource_id")
+            if not dataset_id_list:
+                logging.error(f"Could not find dataset_id in link: {link}")
+                continue
+            dataset_id = dataset_id_list[0]
+
+            dataset_url = f"{self.superset_base_url}/api/v1/dataset/{dataset_id}"
+            logging.info(f"Fetching metadata from {dataset_url}")
+
+            response = self.send_get_request(dataset_url, token=self.jwt_token)
+            if not response:
+                logging.error(f"Failed to fetch metadata for dataset ID {dataset_id}")
+                continue
+            try:
+                metadata_json = response.json()
+                dataset_metadata = metadata_json.get("result", {})
+            except json.JSONDecodeError as e:
+                logging.error(f"Failed to parse metadata JSON for dataset ID {dataset_id}: {e}")
+                continue
+
+            verbose_name = dataset_metadata.get("verbose_name")
+            if not verbose_name or verbose_name == dataset_metadata.get("table_name"):
+                verbose_name = dataset_metadata.get("table_name", "").replace("_", " ").title()
+
+            #Get dataset metadata
+            dataset_info = {
+                "dataset_id": dataset_metadata.get("id"),
+                "catalog": dataset_metadata.get("catalog"),
+                "changed_on": dataset_metadata.get("changed_on"),
+                "table_name": dataset_metadata.get("table_name"),
+                "verbose_name": verbose_name,
+                "schema": dataset_metadata.get("schema"),
+                "datasource_type": dataset_metadata.get("datasource_type"),
+                "description": dataset_metadata.get("description"),
+            }
+
+            #Get column metadata
+            columns = dataset_metadata.get("columns", [])
+            if columns:
+                for col in columns:
+                    record = dataset_info.copy()
+                    record.update({
+                        "column_id": col.get("id"),
+                        "column_name": col.get("column_name"),
+                        "column_verbose_name": col.get("verbose_name") or col.get("column_name", "").replace("_", " ").title(),
+                        "column_type": col.get("type"),
+                        "groupby": col.get("groupby"),
+                        "filterable": col.get("filterable"),
+                        "is_dttm": col.get("is_dttm"),
+                        "column_description": col.get("description"),
+                    })
+                    records.append(record)
+            else:
+                records.append(dataset_info)
+
+        #Write to CSV
+        if records:
+            fieldnames = [
+                "dataset_id", "table_name", "verbose_name", "schema", "catalog",
+                "datasource_type", "changed_on", "description",
+                "column_id", "column_name", "column_verbose_name", "column_type",
+                "groupby", "filterable", "is_dttm", "column_description"
+            ]
+            fieldnames = [field for field in fieldnames if any(field in rec for rec in records)]
+
+            with open(output_file, mode='w', newline='', encoding='utf-8') as csvfile:
+                writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
+                writer.writeheader()
+                writer.writerows(records)
+
+            logging.info(f"Dataset metadata exported to {output_file}")
+        else:
+            logging.warning("No dataset metadata exported.")
+
+    def get_dataset_tables_dict(self, tables=[]) -> dict:
+        if not self.jwt_token:
+            self.superset_authenticate()
+
+        dataset_links = self.get_dataset_links()
+        if not dataset_links:
+            logging.error("No dataset links found.")
+            return {}
+
+        self.get_dataset_data(dataset_links)
+
+        # Load data from the CSV file (generated by get_dataset_data)
+        records = []
+        try:
+            with open("dataset_metadata.csv", mode="r", encoding="utf-8") as f:
+                reader = csv.DictReader(f)
+                for row in reader:
+                    records.append(row)
+        except Exception as e:
+            logging.error(f"Failed to load dataset_metadata.csv: {e}")
+            return {}
+
+        # Group column data by table name
+        table_dict = {}
+        for row in records:
+            table_name = row.get("table_name")
+            if tables and table_name not in tables:
+                continue
+
+            if table_name not in table_dict:
+                table_dict[table_name] = []
+
+            table_dict[table_name].append({
+                "column_name": row.get("column_name", ""),
+                "type": row.get("column_type", ""),
+                "label": row.get("column_verbose_name") or row.get("column_name", "").replace("_", " ").title(),
+                "description": row.get("column_description", "")
+            })
+
+        # Add markdown for each table
+        for table_name, cols in table_dict.items():
+            md_parts = []
+            md_parts.append(f"## `{table_name}` Columns\n\n")
+            md_parts.append("| Column Name | Label | Type | Description | Like-compliant |\n")
+            md_parts.append("|-------------|-------|------|-------------|----------------|\n")
+            for col in cols:
+                column_name = col["column_name"]
+                label = col["label"]
+                col_type = col["type"].upper()
+                description = col["description"]
+                expr = row.get("expression", "") or ""
+                if col_type == "STRING" and (" in (" in expr.lower() or " in(" in expr.lower()):
+                    like_compliant = False
+                else:
+                    like_compliant = True
+                md_parts.append(
+                    f"| `{column_name}` | {label} | {col_type} | {description} | {str(like_compliant)} |\n"
+                )
+            md_parts.append("\n")
+            md_str = "".join(md_parts)
+            # Add markdown string to the dict for this table
+            # If table_dict[table_name] is a list, convert to dict with 'columns' and 'md'
+            table_dict[table_name] = {
+                "columns": cols,
+                "md": md_str
+            }
+        return table_dict
+
+
+    def get_dataset_tables_md(self, tables=[]) -> str:
+        if not self.jwt_token:
+            self.superset_authenticate()
+
+        dataset_links = self.get_dataset_links()
+        if not dataset_links:
+            logging.error("No dataset links found.")
+            return ""
+
+        self.get_dataset_data(dataset_links)
+
+        # Load data from the CSV file
+        records = []
+        try:
+            with open("dataset_metadata.csv", mode="r", encoding="utf-8") as f:
+                reader = csv.DictReader(f)
+                for row in reader:
+                    records.append(row)
+        except Exception as e:
+            logging.error(f"Failed to load dataset_metadata.csv: {e}")
+            return ""
+
+        # Group columns by table_name
+        table_columns = {}
+        for row in records:
+            table_name = row.get("table_name")
+            if tables and table_name not in tables:
+                continue
+
+            if table_name not in table_columns:
+                table_columns[table_name] = []
+
+            column_name = row.get("column_name", "")
+            col_type = row.get("column_type", "")
+            col_type = row.get("column_type", "").upper()
+            expr = row.get("expression", "") or ""
+
+            if col_type == "STRING" and (" in (" in expr.lower() or " in(" in expr.lower()):
+                like_compliant = False
+            else:
+                like_compliant = True
+
+
+            table_columns[table_name].append({
+                "column_name": column_name,
+                "label": row.get("column_verbose_name") or column_name.replace("_", " ").title(),
+                "type": col_type,
+                "description": row.get("column_description", ""),
+                "like_compliant": str(like_compliant)
+            })
+
+        # Build markdown
+        md_parts = []
+        for table_name, cols in table_columns.items():
+            md_parts.append(f"## `{table_name}` Columns\n\n")
+            md_parts.append("| Column Name | Label | Type | Description | Like-compliant |\n")
+            md_parts.append("|-------------|-------|------|-------------|----------------|\n")
+
+            for col in cols:
+                md_parts.append(
+                    f"| `{col['column_name']}` | {col['label']} | {col['type']} | "
+                    f"{col['description']} | {col['like_compliant']} |\n"
+                )
+            md_parts.append("\n")
+
+        markdown_output = "".join(md_parts)
+
+        # Write to file
+        with open("dataset_tables.md", "w", encoding="utf-8") as f:
+            f.write(markdown_output)
+
+        logging.info("Saved table metadata Markdown to dataset_tables.md")
+        return markdown_output
+    
+    def get_dataset_prompt(self, dataset, prompt_template=None):
+        """
+        Returns a prompt string for the given dataset, using a template.
+        - dataset: table name to fetch from docs/dataset_tables.json
+        - prompt_template: template string with '{table}' placeholder, or None to load from docs/<dataset>-template.md
+        """
+        import os
+        import json
+        # Load table data from docs/dataset_tables.json
+        table_json_path = os.path.join("docs", "dataset_tables.json")
+        if not os.path.exists(table_json_path):
+            raise FileNotFoundError(f"dataset_tables.json not found at {table_json_path}")
+        with open(table_json_path, "r", encoding="utf-8") as f:
+            tables = json.load(f)
+        if dataset not in tables:
+            raise ValueError(f"Dataset '{dataset}' not found in dataset_tables.json")
+        table_md = tables[dataset].get("md")
+        if not table_md:
+            raise ValueError(f"No markdown found for dataset '{dataset}' in dataset_tables.json")
+        # Load template
+        if prompt_template is None:
+            template_filename = f"{dataset.replace(' ', '_')}-template.md"
+            template_path = os.path.join("docs", template_filename)
+            if os.path.exists(template_path):
+                with open(template_path, "r", encoding="utf-8") as f:
+                    prompt_template = f.read()
+            else:
+                prompt_template = "{table}"
+        # Build prompt
+        prompt = prompt_template.replace("{table}", table_md)
+        return prompt
\ No newline at end of file
